#include "utils.h"
#include "layout.h"
#include "atomic.h"

Atom wm_atoms[WMLast];
Atom net_atoms[NetLast];

char* wm_strings[WMLast]={
  [WMProtocols]="WM_PROTOCOLS",
  [WMDelete]="WM_DELETE_WINDOW",
  [WMState]="WM_STATE",
  [WMName]="WM_NAME",
  [WMWindowRole]="WM_WINDOW_ROLE",
  [WMClass]="WM_CLASS",
  [WMTakeFocus]="WM_TAKE_FOCUS"
};

char* net_strings[NetLast]={
  [NetSupported]="_NET_SUPPORTED",
  [NetWMName]="_NET_WM_NAME",
  [NetWMState]="_NET_WM_STATE",
  [NetWMFullscreen]="_NET_WM_STATE_FULLSCREEN",
  [NetWMMaximizedVert]="_NET_WM_STATE_MAXIMIZED_VERT",
  [NetWMMaximizedHorz]="_NET_WM_STATE_MAXIMIZED_HORZ",
  [NetWMHidden]="_NET_WM_STATE_HIDDEN",
  [NetWMSkipTaskbar]="_NET_WM_STATE_SKIP_TASKBAR",
  [NetCloseWindow]="_NET_CLOSE_WINDOW",
  [NetActiveWindow]="_NET_ACTIVE_WINDOW",
  [NetDesktopGeometry]="_NET_DESKTOP_GEOMETRY",
  [NetDesktopViewport]="_NET_DESKTOP_VIEWPORT",
  [NetClientList]="_NET_CLIENT_LIST",
  [NetClientListStacking]="_NET_CLIENT_LIST_STACKING",
  [NetWMAllowedActions]="_NET_WM_ALLOWED_ACTIONS",
  [NetWMActionFullscreen]="_NET_WM_ACTION_FULLSCREEN",
  [NetWMActionMaximizeHorz]="_NET_WM_ACTION_MAXIMIZE_HORZ",
  [NetWMActionMaximizeVert]="_NET_WM_ACTION_MAXIMIZE_VERT",
  [NetWMActionClose]="_NET_WM_ACTION_CLOSE",
  [NetWMWindowType]="_NET_WM_WINDOW_TYPE",
  [NetWMWindowTypeDesktop]="_NET_WM_WINDOW_TYPE_DESKTOP",
  [NetWMWindowTypeDock]="_NET_WM_WINDOW_TYPE_DOCK",
  [NetWMWindowTypeToolbar]="_NET_WM_WINDOW_TYPE_TOOLBAR",
  [NetWMWindowTypeMenu]="_NET_WM_WINDOW_TYPE_MENU",
  [NetWMWindowTypeUtility]="_NET_WM_WINDOW_TYPE_UTILITY",
  [NetWMWindowTypeSplash]="_NET_WM_WINDOW_TYPE_SPLASH",
  [NetWMWindowTypeDialog]="_NET_WM_WINDOW_TYPE_DIALOG",
  [NetWMWindowTypeNormal]="_NET_WM_WINDOW_TYPE_NORMAL",
  [NetWMDesktop]="_NET_WM_DESKTOP",
  [NetNumberOfDesktops]="_NET_NUMBER_OF_DESKTOPS",
  [NetCurrentDesktop]="_NET_CURRENT_DESKTOP",
  [NetDesktopNames]="_NET_DESKTOP_NAMES",
  [NetSupportingWMCheck]="_NET_SUPPORTING_WM_CHECK",
  [NetWMPid]="_NET_WM_PID"
};

long atomic_getprop_long(Window w, Atom a){
  int format;
  long result = -1;
  unsigned char *p = 0;
  unsigned long n, extra;
  Atom real;
  if(XGetWindowProperty(XSTATE.dpy, w, a, 0, 1, False, XA_CARDINAL, &real, &format, &n, &extra, (unsigned char **)&p)!=Success)return -1;
  if(n != 0){
    result=*((long*)p);
  }
  XFree(p);
  return result;
}

long* atomic_getprop_longa(Window w, Atom a, int* n){
  int format;
  unsigned char *p = 0;
  unsigned long nn, extra;
  Atom real;
  if(XGetWindowProperty(XSTATE.dpy, w, a, 0, 0x7fffffff, False, XA_CARDINAL, &real, &format, &nn, &extra, (unsigned char **)&p)!=Success){*n=0;return 0;}
  *n=nn;
  return (long*)p;
}

Window atomic_getprop_window(Window w, Atom a){
  int format;
  Window result = -1;
  unsigned char *p = 0;
  unsigned long n, extra;
  Atom real;
  if(XGetWindowProperty(XSTATE.dpy, w, a, 0, 1, False, XA_WINDOW, &real, &format, &n, &extra, (unsigned char **)&p)!=Success)return -1;
  if(n != 0){
    result=*((Window*)p);
  }
  XFree(p);
  return result;
}

Window* atomic_getprop_windowa(Window w, Atom a, int* n){
  int format;
  unsigned char *p = 0;
  unsigned long nn, extra;
  Atom real;
  if(XGetWindowProperty(XSTATE.dpy, w, a, 0, 0x7fffffff, False, XA_WINDOW, &real, &format, &nn, &extra, (unsigned char **)&p)!=Success){*n=0;return 0;}
  *n=nn;
  return (Window*)p;
}

Atom atomic_getprop_atom(Window w, Atom a){
  int format;
  Atom result = -1;
  unsigned char *p = 0;
  unsigned long n, extra;
  Atom real;
  if(XGetWindowProperty(XSTATE.dpy, w, a, 0L, 1, False, XA_ATOM, &real, &format, &n, &extra, (unsigned char **)&p)!=Success)return -1;
  if(n != 0){
    result=*((Atom*)p);
  }
  XFree(p);
  return result;
}

Atom* atomic_getprop_atoma(Window w, Atom a, int* n){
  int format;
  unsigned char *p = 0;
  unsigned long nn, extra;
  Atom real;
  if(XGetWindowProperty(XSTATE.dpy, w, a, 0, 0x7fffffff, False, XA_ATOM, &real, &format, &nn, &extra, (unsigned char **)&p)!=Success){*n=0;return 0;}
  *n=nn;
  return (Atom*)p;
}


void atomic_getstring(Window w, Atom a, char* str){
  XTextProperty tp;
  if (XGetTextProperty(XSTATE.dpy, w, &tp, a) != 0){
    if (tp.encoding == XA_STRING){
      /*
      int n;
      char** cl;
      if (Success <= XTextPropertyToStringList(&tp, &cl, &n) && cl && *cl && n > 0){
	//take only one for now
	strncpy(str, cl[0],256);
	str[255]=0;
	XFreeStringList(cl);
      }
      */

      strncpy(str, (char*)tp.value,256);
      str[255]=0;
    }else{
      int n;
      char** cl;
      if (Success <= Xutf8TextPropertyToTextList(XSTATE.dpy, &tp, &cl, &n) && cl && *cl && n > 0){
	//take only one for now
	strncpy(str, cl[0],256);
	str[255]=0;
	XFreeStringList(cl);
      }
    }
    XFree(tp.value);
  }
}

void atomic_setprop_long(Window w, Atom a, long data){
  XChangeProperty(XSTATE.dpy, w, a, XA_CARDINAL, 32, PropModeReplace, (unsigned char *)&data, 1);
}

void atomic_setprop_longa(Window w, Atom a, long* data, int n){
  XChangeProperty(XSTATE.dpy, w, a, XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data, n);
}

void atomic_setprop_window(Window w, Atom a, Window data){
  XChangeProperty(XSTATE.dpy, w, a, XA_WINDOW, 32, PropModeReplace, (unsigned char *)&data, 1);
}

void atomic_setprop_windowa(Window w, Atom a, Window* data, int n){
  XChangeProperty(XSTATE.dpy, w, a, XA_WINDOW, 32, PropModeReplace, (unsigned char *)data, n);
}

void atomic_setprop_atom(Window w, Atom a, Atom data){
  XChangeProperty(XSTATE.dpy, w, a, XA_ATOM, 32, PropModeReplace, (unsigned char *)&data, 1);
}

void atomic_setprop_atoma(Window w, Atom a, Atom* data, int n){
  XChangeProperty(XSTATE.dpy, w, a, XA_ATOM, 32, PropModeReplace, (unsigned char *)data, n);
}

void atomic_setstrings(Window w, Atom a, char** str, int count){
  XTextProperty xtp;
#ifdef X_HAVE_UTF8_STRING
  int status=Xutf8TextListToTextProperty(XSTATE.dpy, str, count, XUTF8StringStyle, &xtp);
  if (status != XNoMemory && status != XLocaleNotSupported) {
    XSetTextProperty(XSTATE.dpy,w, &xtp, a);
    XFree(xtp.value);
  }
#else
  if (XStringListToTextProperty(str, count, &xtp)) {
    XSetTextProperty(XSTATE.dpy,w, &xtp, a);
    XFree(xtp.value);
  }
#endif
}

void atomic_sendmsg(Window w, Atom a, Time t) {
  XEvent ce;
  ce.xclient.type = ClientMessage;
  ce.xclient.message_type = wm_atoms[WMProtocols];
  ce.xclient.display = XSTATE.dpy;
  ce.xclient.window = w;
  ce.xclient.format = 32;
  ce.xclient.data.l[0] = a;
  ce.xclient.data.l[1] = t;
  ce.xclient.data.l[2] = 0l;
  ce.xclient.data.l[3] = 0l;
  ce.xclient.data.l[4] = 0l;
  XSendEvent(XSTATE.dpy, w, False, NoEventMask, &ce);
}
